@@ -2,8 +2,8 @@ using System;
 using System.Reflection;
 using System.Runtime.CompilerServices;
 
-[assembly: ApplicationVersion ("1.9.5", "1.9.5")]
-[assembly: ApplicationBuildInformation ("git-checkout", "linux-gnu", "x86_64", "2011-03-09 10:46:24 CST")]
+[assembly: ApplicationVersion ("1.9.6", "1.9.6")]
+[assembly: ApplicationBuildInformation ("git-checkout", "freebsd8.2", "amd64", "2011-03-24 09:50:43 MYT")]
 
 [assembly: AssemblyVersion ("1.9.0.0")]
 [assembly: AssemblyTitle ("Banshee")]
