@@ -92,9 +92,9 @@ namespace Banshee.ServiceStack
 #if WIN32
             // There are two sets of environement variables we need to impact with our LANG.
             // refer to : http://article.gmane.org/gmane.comp.gnu.mingw.user/8272
-            CultureInfo current_ci = CultureInfo.CurrentUICulture;
-            string env = String.Concat ("LANG=", current_ci.TwoLetterISOLanguageName);
-            Environment.SetEnvironmentVariable ("LANG", current_ci.TwoLetterISOLanguageName);
+            var lang_code = CultureInfo.CurrentUICulture.TwoLetterISOLanguageName;
+            string env = String.Concat ("LANG=", lang_code);
+            Environment.SetEnvironmentVariable ("LANG", lang_code);
             _putenv (env);
 #endif
 
@@ -169,6 +169,19 @@ namespace Banshee.ServiceStack
             }
         }
 
+        static bool paths_initialized;
+        public static void InitializePaths ()
+        {
+            if (!paths_initialized) {
+                // We changed banshee-1 to banshee everywhere except the
+                // ~/.config/banshee-1/ and ~/.cache/banshee-1 directories, and 
+                // for gconf
+                Paths.UserApplicationName = "banshee-1";
+                Paths.ApplicationName = InternalName;
+                paths_initialized = true;
+            }
+        }
+
         [DllImport ("libglib-2.0-0.dll")]
         static extern IntPtr g_get_language_names ();
 
@@ -192,7 +205,7 @@ namespace Banshee.ServiceStack
             }
 
             if (!shown) {
-                Banshee.Web.Browser.Open (String.Format ("http://library.gnome.org/users/{0}/{1}/", project, Version));
+                Banshee.Web.Browser.Open (String.Format ("http://library.gnome.org/users/{0}/", project));
             }
         }
 
@@ -279,7 +292,7 @@ namespace Banshee.ServiceStack
         }
 
         public static string InternalName {
-            get { return "banshee-1"; }
+            get { return "banshee"; }
         }
 
         public static string IconName {
