@@ -49,6 +49,7 @@ namespace Banshee.Collection.Gui
     {
         private Dictionary<int, SurfaceCache> scale_caches  = new Dictionary<int, SurfaceCache> ();
         private HashSet<int> cacheable_cover_sizes = new HashSet<int> ();
+        private HashSet<string> null_artwork_ids = new HashSet<string> ();
 
         private class SurfaceCache : LruCache<string, Cairo.ImageSurface>
         {
@@ -71,6 +72,7 @@ namespace Banshee.Collection.Gui
             AddCachedSize (42);
             AddCachedSize (48);
             AddCachedSize (64);
+            AddCachedSize (90);
             AddCachedSize (300);
 
             try {
@@ -78,6 +80,18 @@ namespace Banshee.Collection.Gui
             } catch (Exception e) {
                 Log.Exception ("Could not migrate album artwork cache directory", e);
             }
+
+            Banshee.Metadata.MetadataService.Instance.ArtworkUpdated += OnArtworkUpdated;
+        }
+
+        public void Dispose ()
+        {
+            Banshee.Metadata.MetadataService.Instance.ArtworkUpdated -= OnArtworkUpdated;
+        }
+
+        private void OnArtworkUpdated (IBasicTrackInfo track)
+        {
+            ClearCacheFor (track.ArtworkId, true);
         }
 
         public Cairo.ImageSurface LookupSurface (string id)
@@ -103,8 +117,13 @@ namespace Banshee.Collection.Gui
                 return surface;
             }
 
+            if (null_artwork_ids.Contains (id)) {
+                return null;
+            }
+
             Pixbuf pixbuf = LookupScalePixbuf (id, size);
             if (pixbuf == null || pixbuf.Handle == IntPtr.Zero) {
+                null_artwork_ids.Add (id);
                 return null;
             }
 
@@ -122,11 +141,8 @@ namespace Banshee.Collection.Gui
                     int bytes = 4 * size * size;
                     int max = (1 << 20) / bytes;
 
-                    Log.DebugFormat ("Creating new surface cache for {0}px, {1} KB (max) images, capped at 1 MB ({2} items)",
-                        size, bytes, max);
-
-                    cache = new SurfaceCache (max);
-                    scale_caches.Add (size, cache);
+                    ChangeCacheSize (size, max);
+                    cache = scale_caches[size];
                 }
 
                 cache.Add (id, surface);
@@ -147,12 +163,17 @@ namespace Banshee.Collection.Gui
                 return null;
             }
 
+            if (null_artwork_ids.Contains (id)) {
+                return null;
+            }
+
             // Find the scaled, cached file
             string path = CoverArtSpec.GetPathForSize (id, size);
             if (File.Exists (new SafeUri (path))) {
                 try {
                     return new Pixbuf (path);
                 } catch {
+                    null_artwork_ids.Add (id);
                     return null;
                 }
             }
@@ -169,6 +190,7 @@ namespace Banshee.Collection.Gui
                         Pixbuf pixbuf = new Pixbuf (unconverted_path);
                         if (pixbuf.Width < 50 || pixbuf.Height < 50) {
                             Hyena.Log.DebugFormat ("Ignoring cover art {0} because less than 50x50", unconverted_path);
+                            null_artwork_ids.Add (id);
                             return null;
                         }
 
@@ -184,7 +206,18 @@ namespace Banshee.Collection.Gui
             if (orig_exists && size >= 10) {
                 try {
                     Pixbuf pixbuf = new Pixbuf (orig_path);
-                    Pixbuf scaled_pixbuf = pixbuf.ScaleSimple (size, size, Gdk.InterpType.Bilinear);
+
+                    // Make it square if width and height difference is within 20%
+                    const double max_ratio = 1.2;
+                    double ratio = (double)pixbuf.Height / pixbuf.Width;
+                    int width = size, height = size;
+                    if (ratio > max_ratio) {
+                        width = (int)Math.Round (size / ratio);
+                    }else if (ratio < 1d / max_ratio) {
+                        height = (int)Math.Round (size * ratio);
+                    }
+
+                    Pixbuf scaled_pixbuf = pixbuf.ScaleSimple (width, height, Gdk.InterpType.Bilinear);
 
                     if (IsCachedSize (size)) {
                         Directory.Create (System.IO.Path.GetDirectoryName (path));
@@ -198,16 +231,32 @@ namespace Banshee.Collection.Gui
                 } catch {}
             }
 
+            null_artwork_ids.Add (id);
             return null;
         }
 
         public void ClearCacheFor (string id)
         {
+            ClearCacheFor (id, false);
+        }
+
+        public void ClearCacheFor (string id, bool inMemoryCacheOnly)
+        {
+            if (String.IsNullOrEmpty (id)) {
+                return;
+            }
+
             // Clear from the in-memory cache
             foreach (int size in scale_caches.Keys) {
                 scale_caches[size].Remove (id);
             }
 
+            null_artwork_ids.Remove (id);
+
+            if (inMemoryCacheOnly) {
+                return;
+            }
+
             // And delete from disk
             foreach (int size in CachedSizes ()) {
                 var uri = new SafeUri (CoverArtSpec.GetPathForSize (id, size));
@@ -232,6 +281,24 @@ namespace Banshee.Collection.Gui
             return cacheable_cover_sizes;
         }
 
+        public void ChangeCacheSize (int size, int max_count)
+        {
+            SurfaceCache cache;
+            if (scale_caches.TryGetValue (size, out cache)) {
+                if (max_count > cache.MaxCount) {
+                    Log.DebugFormat (
+                        "Growing surface cache for {0}px images to {1:0.00} MiB ({2} items)",
+                        size, 4 * size * size * max_count / 1048576d, max_count);
+                    cache.MaxCount = max_count;
+                }
+            } else {
+                Log.DebugFormat (
+                    "Creating new surface cache for {0}px images, capped at {1:0.00} MiB ({2} items)",
+                    size, 4 * size * size * max_count / 1048576d, max_count);
+                scale_caches.Add (size, new SurfaceCache (max_count));
+            }
+        }
+
         private static int dispose_count = 0;
         public static void DisposePixbuf (Pixbuf pixbuf)
         {
@@ -266,7 +333,7 @@ namespace Banshee.Collection.Gui
             var legacy_root_path = CoverArtSpec.LegacyRootPath;
 
             if (version < 1) {
-                string legacy_artwork_path = Paths.Combine (Paths.LegacyApplicationData, "covers");
+                string legacy_artwork_path = Paths.Combine (LegacyPaths.ApplicationData, "covers");
 
                 if (!Directory.Exists (legacy_root_path)) {
                     Directory.Create (legacy_root_path);
